@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
+#include <openssl/rand.h>
 #include "crypto_backend.h"
 
 static int crypto_backend_initialised = 0;
@@ -52,7 +53,6 @@ int crypt_backend_init(struct crypt_device *ctx)
 		return 0;
 
 	OpenSSL_add_all_digests();
-	log_dbg("OpenSSL crypto backend initialized.");
 
 	crypto_backend_initialised = 1;
 	return 0;
@@ -63,6 +63,11 @@ uint32_t crypt_backend_flags(void)
 	return 0;
 }
 
+const char *crypt_backend_version(void)
+{
+	return SSLeay_version(SSLEAY_VERSION);
+}
+
 /* HASH */
 int crypt_hash_size(const char *name)
 {
@@ -213,3 +218,15 @@ int crypt_hmac_destroy(struct crypt_hmac *ctx)
 	free(ctx);
 	return 0;
 }
+
+/* RNG */
+int crypt_backend_rng(char *buffer, size_t length, int quality, int fips)
+{
+	if (fips)
+		return -EINVAL;
+
+	if (RAND_bytes((unsigned char *)buffer, length) != 1)
+		return -EINVAL;
+
+	return 0;
+}
