@@ -21,13 +21,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/ioctl.h>
-#include <sys/mman.h>
-#include <sys/utsname.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -36,54 +29,46 @@
 #include "luks.h"
 #include "internal.h"
 
-#define div_round_up(a,b) ({          \
-	typeof(a) __a = (a);          \
-	typeof(b) __b = (b);          \
-	(__a - 1) / __b + 1;          \
-})
-
-static inline int round_up_modulo(int x, int m) {
-	return div_round_up(x, m) * m;
-}
-
 static const char *cleaner_name=NULL;
 static uint64_t cleaner_size = 0;
 static int devfd=-1;
 
 static int setup_mapping(const char *cipher, const char *name,
-			 const char *device,
-			 struct volume_key *vk,
+			 unsigned int bsize, struct volume_key *vk,
 			 unsigned int sector, size_t srcLength,
 			 int mode, struct crypt_device *ctx)
 {
-	int device_sector_size = sector_size_for_device(device);
+	struct device *device = crypt_metadata_device(ctx);
 	struct crypt_dm_active_device dmd = {
-		.device = device,
-		.cipher = cipher,
+		.target = DM_CRYPT,
 		.uuid   = NULL,
-		.vk     = vk,
-		.offset = sector,
-		.iv_offset = 0,
-		.size   = 0,
-		.flags  = 0
+		.size   = size_round_up(srcLength, bsize) / SECTOR_SIZE,
+		.flags  = CRYPT_ACTIVATE_PRIVATE,
+		.data_device = device,
+		.u.crypt = {
+			.cipher = cipher,
+			.vk     = vk,
+			.offset = sector,
+			.iv_offset = 0,
+		}
 	};
+	int r;
 
-	dmd.flags = CRYPT_ACTIVATE_PRIVATE;
 	if (mode == O_RDONLY)
 		dmd.flags |= CRYPT_ACTIVATE_READONLY;
-	/*
-	 * we need to round this to nearest multiple of the underlying
-	 * device's sector size, otherwise the mapping will be refused.
-	 */
-	if(device_sector_size < 0) {
-		log_err(ctx, _("Unable to obtain sector size for %s"), device);
-		return -EINVAL;
+
+	r = device_block_adjust(ctx, dmd.data_device, DEV_OK,
+				dmd.u.crypt.offset, &dmd.size, &dmd.flags);
+	if (r < 0)
+		return r;
+
+	if (mode != O_RDONLY && dmd.flags & CRYPT_ACTIVATE_READONLY) {
+		log_err(ctx, _("Cannot write to device %s, permission denied.\n"),
+			device_path(device));
+		return -EACCES;
 	}
-
-	dmd.size = round_up_modulo(srcLength,device_sector_size)/SECTOR_SIZE;
 	cleaner_size = dmd.size;
-
-	return dm_create_device(name, "TEMP", &dmd, 0);
+	return dm_create_device(ctx, name, "TEMP", &dmd, 0);
 }
 
 static void sigint_handler(int sig __attribute__((unused)))
@@ -92,7 +77,7 @@ static void sigint_handler(int sig __attribute__((unused)))
 		close(devfd);
 	devfd = -1;
 	if(cleaner_name)
-		dm_remove_device(cleaner_name, 1, cleaner_size);
+		dm_remove_device(NULL, cleaner_name, 1, cleaner_size);
 
 	signal(SIGINT, SIG_DFL);
 	kill(getpid(), SIGINT);
@@ -113,9 +98,8 @@ static const char *_error_hint(char *cipherMode, size_t keyLength)
 static int LUKS_endec_template(char *src, size_t srcLength,
 			       struct luks_phdr *hdr,
 			       struct volume_key *vk,
-			       const char *device,
 			       unsigned int sector,
-			       ssize_t (*func)(int, void *, size_t),
+			       ssize_t (*func)(int, int, void *, size_t),
 			       int mode,
 			       struct crypt_device *ctx)
 {
@@ -124,10 +108,14 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 	char *dmCipherSpec = NULL;
 	const char *dmDir = dm_get_dir();
 	int r = -1;
+	int bsize = device_block_size(crypt_metadata_device(ctx));
+
+	if (bsize <= 0)
+		return -EINVAL;
 
 	if(dmDir == NULL) {
 		log_err(ctx, _("Failed to obtain device mapper directory."));
-		return -1;
+		return -EINVAL;
 	}
 	if(asprintf(&name,"temporary-cryptsetup-%d",getpid())               == -1 ||
 	   asprintf(&fullpath,"%s/%s",dmDir,name)                           == -1 ||
@@ -139,12 +127,12 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 	signal(SIGINT, sigint_handler);
 	cleaner_name = name;
 
-	r = setup_mapping(dmCipherSpec, name, device,
-			  vk, sector, srcLength, mode, ctx);
+	r = setup_mapping(dmCipherSpec, name, bsize, vk, sector, srcLength, mode, ctx);
 	if(r < 0) {
-		log_err(ctx, _("Failed to setup dm-crypt key mapping for device %s.\n"
+		if (r != -EACCES && r != -ENOTSUP)
+			log_err(ctx, _("Failed to setup dm-crypt key mapping for device %s.\n"
 			"Check that kernel supports %s cipher (check syslog for more info).\n%s"),
-			device, dmCipherSpec,
+			device_path(crypt_metadata_device(ctx)), dmCipherSpec,
 			_error_hint(hdr->cipherMode, vk->keylength * 8));
 		r = -EIO;
 		goto out1;
@@ -157,7 +145,7 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 		goto out2;
 	}
 
-	r = func(devfd,src,srcLength);
+	r = func(devfd, bsize, src, srcLength);
 	if(r < 0) {
 		log_err(ctx, _("Failed to access temporary keystore device.\n"));
 		r = -EIO;
@@ -169,7 +157,7 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 	close(devfd);
 	devfd = -1;
  out2:
-	dm_remove_device(cleaner_name, 1, cleaner_size);
+	dm_remove_device(ctx, cleaner_name, 1, cleaner_size);
  out1:
 	signal(SIGINT, SIG_DFL);
 	cleaner_name = NULL;
@@ -183,21 +171,19 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 int LUKS_encrypt_to_storage(char *src, size_t srcLength,
 			    struct luks_phdr *hdr,
 			    struct volume_key *vk,
-			    const char *device,
 			    unsigned int sector,
 			    struct crypt_device *ctx)
 {
-	return LUKS_endec_template(src,srcLength,hdr,vk, device,
-				   sector, write_blockwise, O_RDWR, ctx);
+	return LUKS_endec_template(src, srcLength, hdr, vk, sector,
+				   write_blockwise, O_RDWR, ctx);
 }
 
 int LUKS_decrypt_from_storage(char *dst, size_t dstLength,
 			      struct luks_phdr *hdr,
 			      struct volume_key *vk,
-			      const char *device,
 			      unsigned int sector,
 			      struct crypt_device *ctx)
 {
-	return LUKS_endec_template(dst,dstLength,hdr,vk, device,
-				   sector, read_blockwise, O_RDONLY, ctx);
+	return LUKS_endec_template(dst, dstLength, hdr, vk, sector,
+				   read_blockwise, O_RDONLY, ctx);
 }
