@@ -30,8 +30,12 @@
 #include <sys/types.h>
 #include "utils_dm.h"
 
+/* These are DM helpers used only by this file */
+int dm_is_dm_device(int major, int minor);
+int dm_is_dm_kernel_name(const char *name);
+char *dm_device_path(const char *prefix, int major, int minor);
+
 char *crypt_lookup_dev(const char *dev_id);
-int crypt_sysfs_check_crypt_segment(const char *device, uint64_t offset, uint64_t size);
 int crypt_sysfs_get_rotational(int major, int minor, int *rotational);
 
 static char *__lookup_dev(char *path, dev_t dev, int dir_level, const int max_level)
@@ -135,7 +139,7 @@ char *crypt_lookup_dev(const char *dev_id)
 	if (snprintf(path, sizeof(path), "/sys/dev/block/%s", dev_id) < 0)
 		return NULL;
 
-	len = readlink(path, link, sizeof(link));
+	len = readlink(path, link, sizeof(link) - 1);
 	if (len < 0) {
 		/* Without /sys use old scan */
 		if (stat("/sys/dev/block", &st) < 0)
@@ -168,82 +172,6 @@ char *crypt_lookup_dev(const char *dev_id)
 	return devpath;
 }
 
-static int crypt_sysfs_get_major_minor(const char *kname, int *major, int *minor)
-{
-	char path[PATH_MAX], tmp[64] = {0};
-	int fd, r = 0;
-
-	if (snprintf(path, sizeof(path), "/sys/block/%s/dev", kname) < 0)
-		return 0;
-
-	if ((fd = open(path, O_RDONLY)) < 0)
-		return 0;
-	r = read(fd, tmp, sizeof(tmp));
-	close(fd);
-
-	if (r <= 0)
-		return 0;
-
-	tmp[63] = '\0';
-	if (sscanf(tmp, "%d:%d", major, minor) != 2)
-		return 0;
-
-	return 1;
-}
-
-static int crypt_sysfs_get_holders_dir(const char *device, char *path, int size)
-{
-	struct stat st;
-
-	if (stat(device, &st) < 0 || !S_ISBLK(st.st_mode))
-		return 0;
-
-	if (snprintf(path, size, "/sys/dev/block/%d:%d/holders",
-		     major(st.st_rdev), minor(st.st_rdev)) < 0)
-		return 0;
-
-	return 1;
-}
-
-int crypt_sysfs_check_crypt_segment(const char *device, uint64_t offset, uint64_t size)
-{
-	DIR *dir;
-	struct dirent *d;
-	char path[PATH_MAX], *dmname;
-	int major, minor, r = 0;
-
-	if (!crypt_sysfs_get_holders_dir(device, path, sizeof(path)))
-		return -EINVAL;
-
-	if (!(dir = opendir(path)))
-		return -EINVAL;
-
-	while (!r && (d = readdir(dir))) {
-		if (!strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
-			continue;
-
-		if (!dm_is_dm_kernel_name(d->d_name)) {
-			r = -EBUSY;
-			break;
-		}
-
-		if (!crypt_sysfs_get_major_minor(d->d_name, &major, &minor)) {
-			r = -EINVAL;
-			break;
-		}
-
-		if (!(dmname = dm_device_path(NULL, major, minor))) {
-			r = -EINVAL;
-			break;
-		}
-		r = dm_check_segment(dmname, offset, size);
-		free(dmname);
-	}
-	closedir(dir);
-
-	return r;
-}
-
 int crypt_sysfs_get_rotational(int major, int minor, int *rotational)
 {
 	char path[PATH_MAX], tmp[64] = {0};
